<?php
/**
 * REST API Cart controller
 *
 * Handles requests to the /cart endpoint.
 *
 * @author   QWQ
 * @category API
 * @package  WooCommerce/API
 * @since    2.6.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * REST API Cart controller class.
 *
 * @package WooCommerce/API
 * @extends WC_REST_Posts_Controller
 */
class W2W_REST_Cart_Controller extends W2W_REST_Posts_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'w2w/v1';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = 'cart';

	/**
	 * Initialize orders actions.
	 */
	public function __construct() {
		
	}

	/**
	 * Register the routes for orders.
	 */
	public function register_routes() {

		register_rest_route( $this->namespace, '/' . $this->rest_base . '/add', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'add_item_to_cart' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_cart_totals' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/update_quantity', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'update_cart_item' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/delete', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'remove_cart_item' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/coupon', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_coupons' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			),
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'apply_coupon' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/remove_coupon', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'remove_coupon' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/address', array(
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'set_address' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/shipping', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_shipping_methods' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			),
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'save_shipping_method' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
		
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/payment', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_payment_methods' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			),
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'save_payment_method' ),
				'permission_callback' => array( $this, 'get_items_permissions_check' ),
				'args'                => $this->get_collection_params(),
			)
		) );
	}
	
	/**
	 * Check whether a given request has permission to read order notes.
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return WP_Error|boolean
	 */
	public function get_items_permissions_check( $request ) {
		return true;
	}
	
	/**
	 * Adds an item to the cart.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function add_item_to_cart( $request ) {
		
		if( isset( $request['empty_cart'] ) ) {
			WC()->cart->empty_cart();
		}
		
		$product_id        = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $request['product_id'] ) );
		$quantity          = empty( $request['quantity'] ) ? 1 : wc_stock_amount( $request['quantity'] );
		$variation_id	   = $request['variation_id'];
		$variation         = $request['variation'];
	
		WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variation );
		do_action( 'woocommerce_ajax_added_to_cart', $product_id );

		return $this->_get_cart_totals( $request );
	}
	
	/**
	 * Update cart item quantity.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function update_cart_item( $request ) {
		$cart_key        = $request['cart_key'];
		
		if ( ! WC()->cart->is_empty() ) {
			
			foreach ( WC()->cart->get_cart() as $cart_item_key => $values ) {
				
				if( $cart_item_key == $cart_key ) {
					
					$_product = $values['data'];

					// Sanitize.
					$quantity = apply_filters( 'woocommerce_stock_amount_cart_item', wc_stock_amount( preg_replace( '/[^0-9\.]/', '', $request['quantity'] ) ), $cart_item_key );

					if ( '' === $quantity || $quantity === $values['quantity'] ) {
						break;
					}

					// Update cart validation.
					$passed_validation = apply_filters( 'woocommerce_update_cart_validation', true, $cart_item_key, $values, $quantity );

					// is_sold_individually.
					if ( $_product->is_sold_individually() && $quantity > 1 ) {
						/* Translators: %s Product title. */
						wc_add_notice( sprintf( __( 'You can only have 1 %s in your cart.', 'woocommerce' ), $_product->get_name() ), 'error' );
						$passed_validation = false;
					}

					if ( $passed_validation ) {
						WC()->cart->set_quantity( $cart_item_key, $quantity, false );
					}
					
					break;
				}
			}
		}
		
		return $this->_get_cart_totals( $request, true );
	}
	
	/**
	 * Removes cart item by cart key.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function remove_cart_item( $request ) {
		$cart_key        = $request['cart_key'];		
		WC()->cart->remove_cart_item( $cart_key );
		
		return $this->_get_cart_totals( $request, true );
	}
	
	
	private function _get_coupons( $coupons ) {
		
		$response = array();
		
		foreach( $coupons as $coupon_code ) {
			
			$coupon = new WC_Coupon( $coupon_code );
			
			$response[] = array(
				'code' => $coupon->get_code(),
				'description' => $coupon->get_description(),
				'discount_type' => $coupon->get_discount_type(),
				'amount' => $coupon->get_amount(),
				'date_expires' => $coupon->get_date_expires() ? $coupon->get_date_expires()->date_i18n('Y.m.d H:i:s') : null,
				'date_created' => $coupon->get_date_created()->date_i18n('Y.m.d H:i:s'),
				'date_modified' => $coupon->get_date_modified()->date_i18n('Y.m.d H:i:s'),
				'usage_count' => $coupon->get_usage_count(),
				'individual_use' => $coupon->get_individual_use(),
				'product_ids' => $coupon->get_product_ids(),
				'excluded_product_ids' => $coupon->get_excluded_product_ids(),
				'usage_limit' => $coupon->get_usage_limit(),
				'usage_limit_per_user' => $coupon->get_usage_limit_per_user(),
				'limit_usage_to_x_items' => $coupon->get_limit_usage_to_x_items(),
				'free_shipping' => $coupon->get_free_shipping(),
				'product_categories' => $coupon->get_product_categories(),
				'excluded_product_categories' => $coupon->get_excluded_product_categories(),
				'exclude_sale_items' => $coupon->get_exclude_sale_items(),
				'minimum_amount' => $coupon->get_minimum_amount(),
				'maximum_amount' => $coupon->get_maximum_amount(),
				'email_restrictions' => $coupon->get_email_restrictions(),
				'used_by' => $coupon->get_used_by(),
				'virtual' => version_compare( WC_VERSION, '3.2', '<' ) ? $coupon->virtual : $coupon->get_virtual()
			);
		}
		
		return apply_filters( 'w2w_rest_coupons_response', $response );
	}
	/**
	 * Get coupon(s) detail.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_coupons( $request ) {

		$applied_coupons = WC()->cart->get_applied_coupons();
		$response = $this->_get_coupons( $applied_coupons );
		
		return $response;
	}
	
	private function _apply_coupon( $request ) {
		
		WC()->cart->add_discount( sanitize_text_field( $request['coupon_code'] ) );
	}
	
	/**
	 * Applies a coupon to the cart.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function apply_coupon( $request ) {
		
		$data = array();
		if( ! wc_coupons_enabled() ) {
			$data['error'] = '暂不支持优惠券';
			$data['coupons'] = array();
			$data['shipping'] = $this->_get_shipping_methods( $request );
			$data['cart'] = $this->_get_cart_totals( $request );
			return $data;
		}
		
		//$this->handle_checkout_params( $request, array( 'address', 'coupon' ) );
		wc_clear_notices();
		
		if ( ! empty( $request['coupon_code'] ) ) {
			$this->_apply_coupon( $request );
		}else{
			WC()->cart->remove_coupons();
			wc_add_notice( __( 'Coupon code removed successfully.', 'woocommerce' ), 'success' );
		}
		
		$data = wc_get_notices();
		wc_clear_notices();
		
		if( isset( $data['error'] ) ) {
			$data['error'] = count( $data['error'] ) > 0 ? html_entity_decode( trim( strip_tags( $data['error'][0] ) ) ) : '错误';
		}
		
		if( isset( $data['success'] ) ) {
			$data['success'] = count( $data['success'] ) > 0 ? html_entity_decode( trim( strip_tags( $data['success'][0] ) ) ) : '成功';
		}
		
		$data['coupons'] = $this->get_coupons( $request );
		
		$data['shipping'] = $this->_get_shipping_methods( $request );
		$data['cart'] = $this->_get_cart_totals( $request );
		return $data;
	}
	
	// 从购物车移除优惠券
	private function _remove_coupon( $request ) {
		
		if( ! empty( $request['coupon_code'] ) ) {
			
			WC()->cart->remove_coupon( $request['coupon_code'] );
			wc_add_notice( __( 'Coupon code removed successfully.', 'woocommerce' ), 'success' );
		}
		
		$data = wc_get_notices();
		wc_clear_notices();
		
		if( isset($data['error'] ) ) {
			$data['error'] = count($data['error']) > 0 ? html_entity_decode( trim( strip_tags( $data['error'][0] ) ) ) : '错误';
		}
		
		if( isset($data['success'] ) ) {
			$data['success'] = count($data['success']) > 0 ? html_entity_decode( trim( strip_tags( $data['success'][0] ) ) ) : '成功';
		}
		
		$data['coupons'] = $this->get_coupons( $request );
		
		$data['shipping'] = $this->_get_shipping_methods( $request );
		$data['cart'] = $this->_get_cart_totals( $request );
		return $data;
	}
	
	public function remove_coupon( $request ) {
		
		return $this->_remove_coupon( $request );
	}
	
	
	private function _get_shipping_methods( $request ) {
		
		if ( ! defined( 'WOOCOMMERCE_CART' ) ) {
			define( 'WOOCOMMERCE_CART', true );
		}
		
		//$this->handle_checkout_params( $request, array( 'address', 'coupon' ) );
		//$this->handle_checkout_params( $request, array( 'address' ) );
		
		WC()->cart->calculate_totals();
		
		$packages = WC()->cart->get_shipping_packages();
		WC()->shipping->calculate_shipping($packages);
		$packages = WC()->shipping->packages;
		
		$response = array();
		if( $packages[0] && $packages[0]['rates'] ) {
			
			foreach( $packages[0]['rates'] as $rate ) {
				
				$response['methods'][] = array(
					'id' => $rate->get_id(),
					'method_id' => $rate->get_method_id(),
					'instance_id' => $rate->get_instance_id(),
					'label' => $rate->get_label(),
					'cost' => floatval( $rate->get_cost() ),
					'taxes' => $rate->get_taxes(),
					'shipping_tax' => $rate->get_shipping_tax()
				);
			}
		}
		else{
			
			$response['methods'] = array();
		}
		
		$response['chosen_method'] = isset( WC()->session->chosen_shipping_methods[ 0 ] ) ? WC()->session->chosen_shipping_methods[ 0 ] : '';
		
		return $response;
	}
	
	/**
	 * Get available shipping methods.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_shipping_methods( $request ) {
		
		$this->_set_address( $request );
		
		return array(
			'coupons' => $this->get_coupons( $request ),
			'shipping' => $this->_get_shipping_methods( $request ),
			'payment' => $this->get_payment_methods( $request ),
			'cart' => $this->_get_cart_totals( $request, true )
		);
	}
	
	private function _save_shipping_method( $request ) {
		
		if ( ! defined( 'WOOCOMMERCE_CART' ) ) {
			define( 'WOOCOMMERCE_CART', true );
		}
		
		$chosen_shipping_methods = WC()->session->get( 'chosen_shipping_methods' );

		/*if ( isset( $request['shipping_method'] ) && is_array( $request['shipping_method'] ) ) {
			foreach ( $request['shipping_method'] as $i => $value ) {
				$chosen_shipping_methods[ $i ] = wc_clean( $value );
			}
		}*/
		
		if ( isset( $request['shipping_method'] ) ) {
			$chosen_shipping_methods[0] = wc_clean( $request['shipping_method'] );
		}

		WC()->session->set( 'chosen_shipping_methods', $chosen_shipping_methods );
	}
	
	/**
	 * Save shipping method.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function save_shipping_method( $request ) {
		
		$this->_save_shipping_method( $request );
		
		return array(
			'shipping' => $this->_get_shipping_methods( $request ),
			'payment' => $this->get_payment_methods( $request ),
			'cart' => $this->_get_cart_totals( $request )
		);
	}
	
	private function _set_address( $request ) {
		
		if ( ! defined( 'WOOCOMMERCE_CART' ) ) {
			define( 'WOOCOMMERCE_CART', true );
		}

		WC()->shipping->reset_shipping();

		$country  = wc_clean( $request['country_id'] );
		$state    = wc_clean( isset( $request['state'] ) ? $request['state'] : '' );
		$postcode = apply_filters( 'woocommerce_shipping_calculator_enable_postcode', true ) ? wc_clean( $request['postcode'] ) : '';
		$city     = apply_filters( 'woocommerce_shipping_calculator_enable_city', false ) ? wc_clean( $request['city'] ) : '';

		if ( $postcode && ! WC_Validation::is_postcode( $postcode, $country ) ) {
			throw new Exception( __( 'Please enter a valid postcode/ZIP.', 'woocommerce' ) );
		} elseif ( $postcode ) {
			$postcode = wc_format_postcode( $postcode, $country );
		}

		if ( $country ) {
			WC()->customer->set_location( $country, $state, $postcode, $city );
			WC()->customer->set_shipping_location( $country, $state, $postcode, $city );
		} else {
			WC()->customer->set_to_base();
			WC()->customer->set_shipping_to_base();
		}

		WC()->customer->calculated_shipping( true );
		
		do_action( 'woocommerce_calculated_shipping' );

		WC()->cart->calculate_totals();
		
		// Check cart items are valid
		//do_action( 'woocommerce_check_cart_items' );
	}
	
	/**
	 * Set customer address.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function set_address( $request ) {
		
		$this->_set_address( $request );
		
		return array(
			'shipping' => $this->_get_shipping_methods( $request ),
			'payment' => $this->get_payment_methods( $request ),
			'cart' => $this->_get_cart_totals( $request )
		);
	}
	
	/**
	 * Get available payment methods.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_payment_methods( $request ) {
		
		$supported_methods = W2W()->util->get_supported_payment_methods();
		
		$response = array();
		if ( WC()->cart->needs_payment() ) {
			
			$payment_methods = WC()->payment_gateways->get_available_payment_gateways();
			$response['methods'] = array();
			$payment_keys = array();
			
			foreach( $payment_methods as $method ) {
				
				foreach( $supported_methods as $payment_method ) {
				
					if( $payment_method == $method->id ) {
						
						$payment = array(
							'id' => $method->id,
							'title' => isset( $method->settings['title'] ) ? $method->settings['title'] : $method->method_title,
							'description' => isset( $method->settings['description'] ) ? $method->settings['description'] : $method->method_description,
							'instructions' => isset( $method->settings['instructions'] ) ? $method->settings['instructions'] : $method->method_description
						);

						$response['methods'][] = $payment;
						$payment_keys[] = $payment['id'];
						break;
					}
				}
			}
			$response['methods'] = apply_filters( 'w2w_get_payment_menthods', $response['methods'] );
			$response['chosen_method'] = WC()->session->get( 'chosen_payment_method' );
			$response['chosen_method'] = ! empty( $response['chosen_method'] ) && in_array( $response['chosen_method'], $payment_keys ) 
										 ? $response['chosen_method']
										 : ( ! empty( $response['methods'] ) ? $response['methods'][0]['id'] : false );
		} else {
			$response['methods'] = array();
			$response['chosen_method'] = '';
		}
		
		return $response;
	}
	
	/**
	 * Save payment method.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function save_payment_method( $request ) {
		if ( ! defined('WOOCOMMERCE_CART') ) {
			define( 'WOOCOMMERCE_CART', true );
		}

		if ( isset( $request['payment_method'] ) ) {
			WC()->session->set( 'chosen_payment_method', $request['payment_method'] );
		}
		
		return array(
			'shipping' => $this->_get_shipping_methods( $request ),
			'payment' => $this->get_payment_methods( $request ),
			'cart' => $this->_get_cart_totals( $request )
		);
	}
	
	private function _get_cart_totals( $request, $check_cart_items = false ) {
		
		$request['check_cart_items'] = $check_cart_items;
		return $this->get_cart_totals( $request );
	}
	
	/**
	 * Get cart items and totals.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function get_cart_totals( $request ) {
		
		if ( ! defined( 'WOOCOMMERCE_CART' ) ) {
			define( 'WOOCOMMERCE_CART', true );
		}

		if( isset( $request['check_cart_items'] ) && $request['check_cart_items'] == true ) {
			do_action( 'woocommerce_check_cart_items' );
		}

		WC()->cart->calculate_totals();
		
		$json = array();
		$temp = WC()->cart->get_cart();
		
		foreach( $temp as $key => $cartitem ) {
			$product_id = $cartitem['variation_id'] > 0 ? $cartitem['variation_id'] : $cartitem['product_id'];
			$post = wc_get_product( $product_id );
			$product = W2W()->util->get_product_data( $post, 'cart' );
			
			$cartitem['product'] = $product;
			
			$json['cart'][$key] = $cartitem;
			
			$json['cart'][$key]['parent_attributes'] = $post->get_type() == 'variation' ? W2W()->util->get_product_attributes( wc_get_product( $json['cart'][$key]['product_id'] ) ) : null;
		}
		
		$cross_sells = WC()->cart->get_cross_sells();
		shuffle( $cross_sells );
		
		$json['cart_total'] = floatval( WC()->cart->total );
		$json['cart_contents_total'] = floatval( WC()->cart->cart_contents_total );
		$json['cart_subtotal'] = WC()->cart->subtotal;
		$json['shipping_total'] = floatval( WC()->cart->shipping_total );
		$json['shipping_tax_total'] = WC()->cart->shipping_tax_total;
		$json['needs_shipping'] = WC()->cart->needs_shipping();
		$json['subtotal_ex_tax'] = WC()->cart->subtotal_ex_tax;
		$json['taxes_total'] = WC()->cart->get_taxes_total();
		$json['cart_contents_count'] = WC()->cart->get_cart_contents_count();
		$json['currency'] = get_woocommerce_currency();
		$json['discount_cart'] = WC()->cart->discount_cart;
		$json['coupon_discount_amounts'] = WC()->cart->coupon_discount_amounts;
		$json['applied_coupons'] = WC()->cart->get_applied_coupons();
		$json['coupons_enabled'] = wc_coupons_enabled();
		$json['cross_sells'] = $cross_sells;
		$json['_wpnonce'] = wp_create_nonce('woocommerce-process_checkout');
		
		// 输出可能的错误
		$all_notices = wc_get_notices();
		wc_clear_notices();
		
		foreach( $all_notices as $type => &$notice ) {
			
			foreach( $notice as &$n ) {
				$n = W2W()->util->strip_html_tags( array( 'a', 'strong' ), html_entity_decode( $n ), false );
			}
		}
		
		$json['errors'] = isset( $all_notices['error'] ) ? $all_notices['error'] : array();
		$json['success'] = isset( $all_notices['success'] ) ? $all_notices['success'] : array();
		$json['notices'] = isset( $all_notices['notice'] ) ? $all_notices['notice'] : array();
		
		return apply_filters( 'w2w_rest_cart_data', $json );
	}
}
